#!/bin/bash
# タグで指定したECRのイメージに、引数で指定した別のタグをつける

set -eu

REPOSITORY=
TARGET_TAG=
TARGET_DIGEST=
SET_TAG=latest

function usage() {
    cat <<EOM
ecr_set_tag.sh
    -r | --repository-name       ECR repository
    -t | --image-tag             target image tag
    -d | --image-digest          target image digest (set 'skip' to skip check)
    -s | --set-tag               tag to set (default: latest)
EOM
    exit 3
}

function parseArg() {
    if [ $# == 0 ]; then usage; fi

    while [[ $# -gt 0 ]]
    do
        key="$1"

        case $key in
            -r|--repository-name)
                REPOSITORY="$2"
                shift
                ;;
            -t|--image-tag)
                TARGET_TAG="$2"
                shift
                ;;
            -d|--image-digest)
                TARGET_DIGEST=$(echo "$2" | tr -d '"' | tr "[:upper:]" "[:lower:]")
                shift
                ;;
            -s|--set-tag)
                SET_TAG="$2"
                shift
                ;;
            *)
                usage
                exit 2
            ;;
        esac
        shift
    done

    if [[ -z "${REPOSITORY}" ]] || [[ -z "${TARGET_TAG}" ]] || [[ -z "${TARGET_DIGEST}" ]]; then
        usage
        exit 2
    fi
}

# 指定イメージタグに別のタグを付与する
# $1: レポジトリ
# $2: イメージタグ
# $3: 付与するタグ
# $4: イメージのdigest
function setTag() {
    local repository_name=$1
    local target_tag=$2
    local set_tag=$3
    local specified_digest=$4
    local actual_digest
    local current_digest
    local manifest

    # デプロイ対象のイメージと現在デプロイされているイメージそれぞれのハッシュ値を取得
    actual_digest=$(aws ecr describe-images --repository-name "${repository_name}" --image-ids "imageTag=${target_tag}" --query "imageDetails[0].imageDigest" | tr -d '"' | tr "[:upper:]" "[:lower:]")
    current_digest=$(aws ecr describe-images --repository-name "${repository_name}" --image-ids "imageTag=${set_tag}" --query "imageDetails[0].imageDigest" | tr -d '"' | tr "[:upper:]" "[:lower:]")

    # デプロイ対象のイメージと指定したイメージが一致するか確認
    if [[ "${specified_digest}" != "skip" ]] && [[ "${specified_digest}" != "${actual_digest}" ]]; then
        echo "image digests do not match: specified:${specified_digest}, actual:${actual_digest}"
        exit 2
    fi

    # 指定したイメージがすでにデプロイされていないか確認
    if [[ "${actual_digest}" = "${current_digest}" ]]; then
        echo "target image '${target_tag}' already tagged as '${set_tag}'"
        exit 0
    fi

    # デプロイ対象のイメージにl指定されたタグを付与
    echo "set tag ${set_tag} to ${target_tag}"
    manifest=$(aws ecr batch-get-image --output text --repository-name "${repository_name}" --image-ids "imageTag=${target_tag}" --query "images[0].imageManifest")
    aws ecr put-image --repository-name "${repository_name}" --image-tag "${set_tag}" --image-manifest "${manifest}"
}

parseArg "$@"
setTag "${REPOSITORY}" "${TARGET_TAG}" "${SET_TAG}" "${TARGET_DIGEST}"
